import requests
import json
from pprint import pprint
import base64
from jinja2 import Template
import codecs
from bs4 import BeautifulSoup
import sys
from datetime import datetime, timezone, timedelta
import sqlite3
import pandas as pd


domain = 'tox1i0lgjy5s'
app_id = 480
api_token = ""
indexName = '一覧2'
DB = 'records.sqlite3'


def encode64(id, pw):
    ostr = '{}:{}'.format(id, pw)
    bstr = base64.b64encode(ostr.encode())
    str = bstr.decode()
    return str


authToken = encode64('18648981@alumni.tus.ac.jp', 'yokomimi01')

headers = {"Content-Type": "application/json",
           "X-Cybozu-Authorization": authToken,
           "Authorization": "Basic {}".format(authToken),
           }


def app_deploy():
    """
    状態を運用開始前から運用中にする
    """

    url = "https://{}.cybozu.com/k/v1/preview/app/deploy.json".format(domain)

    params = {
        "apps": [
            {
                "app": app_id,
            },
        ],
    }

    resp = requests.post(url, json=params, headers=headers)
    dictReturn = json.loads(resp.text)

    pprint(dictReturn)


def getViews():
    """kintoneのビューを取得する"""

    url = f"https://{domain}.cybozu.com/k/v1/app/views.json"
    params = {"app": app_id}

    resp = requests.get(url, json=params, headers=headers)
    dictReturn = json.loads(resp.text)

    return dictReturn


def updateView(views):

    url = f"https://{domain}.cybozu.com/k/v1/preview/app/views.json"
    params = {
        "app": app_id,
        "views": {
            indexName: views,
        },
    }

    resp = requests.put(url, json=params, headers=headers)
    dictReturn = json.loads(resp.text)
    pprint(dictReturn)


def extract_div_content(html_text):
    """HTMLからdivの内容を取得する"""

    soup = BeautifulSoup(html_text, 'html.parser')
    div_content = str(soup.find('div', id='screen'))

    return div_content


def extractHtml(dictView,  html):
    """ビューのHTMLを取得する"""

    view = dictView["views"][indexName]
    view['html'] = html

    return view


def data_select(offset=0):
    """
    kintoneのレコードを取得(再帰的に呼び出す)
    """

    limit = 500
    # domainは適切な値に置き換える必要があります
    url = f"https://{domain}.cybozu.com/k/v1/records.json"

    # fields = ["レコード番号", "製品番号", "製品名", "予定1",
    #             "実績1", "予定2", "実績2", "予定3", "実績3", "処理日"]

    params = {
        "app": app_id,
        # "fields": fields,
        "query": f"order by $id limit {limit} offset {offset}"
    }

    resp = requests.get(url, json=params, headers=headers)
    data = resp.json()
    records = data["records"]

    # もし取得したレコード数がlimitと等しい場合は再帰的に呼び出して残りのレコードを取得
    if len(records) == limit:
        next_records = data_select(offset + limit)
        records.extend(next_records)

    return records


def records2db(records):
    """
    kintoneのレコードを取得してDBに保存する
    """

    # pprint(records)

    if len(records) == 0:
        return

    rec = records[0]
    listType = []
    listField = []
    for key, value in rec.items():

        if value["type"] == "DATE" or value["type"] == "DATETIME" or value["type"] == "CREATED_TIME" or value["type"] == "UPDATED_TIME":
            listType.append('TIMESTAMP')
            listField.append(key)
        elif value["type"] == "NUMBER":
            listType.append('NUMBER')
            listField.append(key)
        elif value["type"] == "RECORD_NUMBER":
            listType.append('NUMBER')
            listField.append(key)
        elif value["type"] == "__ID__":
            listType.append('NUMBER')
            listField.append('ID')
        elif value["type"] == "__REVISION__":
            listType.append('NUMBER')
            listField.append('REVISION')
        else:
            listType.append('TEXT')
            listField.append(key)

    sql = 'CREATE TABLE tbl (\n'
    for fld, tp in zip(listField, listType):
        sql += f'{fld} {tp},\n'

    sql = sql[:-2] + ' );'

    # print(sql)
    # print(listField)
    # print(listType)
    con = sqlite3.connect(DB)
    cur = con.cursor()
    cur.execute('drop table if exists tbl')
    con.commit()

    cur.execute(sql)
    con.commit()

    cur.execute('delete from tbl')
    con.commit()

    # output_data = []
    for entry in records:
        new_entry = {}
        for key, value in entry.items():
            if key == "作成者" or key == "更新者":
                new_entry[key] = value["value"]['name']
            elif key == "更新日時" or key == "作成日時":
                new_entry[key] = convDatetime(value["value"])
            elif key == "登録日":
                new_entry[key] = convDate(value["value"])
            elif key == "$revision":
                new_entry['REVISION'] = int(value["value"])
            elif key == "$id":
                new_entry['ID'] = int(value["value"])
            elif value["type"] == 'DATE':
                new_entry[key] = convDate(value["value"])
            elif value["type"] == 'DATETIME':
                new_entry[key] = convDatetime(value["value"])
            else:
                new_entry[key] = value["value"]

        strFields = ''
        strValues = ''
        for key in listField:
            # print(f'{key} - {new_entry[key]}')
            strFields += f'{key}, '
            strValues += f"'{new_entry[key]}', "

        strFields = strFields[:-2]
        strValues = strValues[:-2]

        # print(strFields)
        # print(strValues)
        sql = f'insert into tbl ({strFields}) values ({strValues})'
        # print(sql)

        cur.execute(sql)
        con.commit()

    con.close()




def convDate(utc_time_str):
    
    if utc_time_str == '' or utc_time_str is None:
        return ''
    
    utc_time = datetime.fromisoformat(utc_time_str.replace("Z", "+00:00"))
    jst_time = utc_time.astimezone(timezone(timedelta(hours=9)))
    return jst_time.strftime('%Y-%m-%d')



def convDatetime(utc_time_str):
    
    if utc_time_str == '' or utc_time_str is None:
        return ''

    utc_time = datetime.fromisoformat(utc_time_str.replace("Z", "+00:00"))
    jst_time = utc_time.astimezone(timezone(timedelta(hours=9)))
    return jst_time.strftime('%Y-%m-%d %H:%M:%S')


# def convDate(iso_string):

#     if iso_string != '' and iso_string is not None:
#         try:
#             input_time = datetime.fromisoformat(iso_string)
#         except ValueError:
#             input_time = datetime.strptime(iso_string, '%Y-%m-%dT%H:%M:%SZ')
#         jst = timezone(timedelta(hours=9))
#         jst_time = input_time.astimezone(jst)

#         return jst_time.strftime('%Y-%m-%d')
#     else:
#         return ''

# def convDatetime(iso_string):

#     if iso_string != '':
#         input_time = datetime.fromisoformat(iso_string)
#         jst = timezone(timedelta(hours=9))
#         jst_time = input_time.astimezone(jst)

#         # Return date and time in string format
#         return jst_time.strftime('%Y-%m-%d %H:%M:%S')
#     else:
#         return ''


def executeSql(sql):

    count = None
    conn = sqlite3.connect(DB)
    cursor = conn.cursor()
    cursor.execute(sql)
    count = cursor.fetchone()[0]
    conn.close()

    return count


if __name__ == "__main__":

    records = data_select()
    records2db(records)

    dictSql = {}
    dictSql['全件数'] = executeSql('select count(*) from tbl')
    dictSql['K50'] = executeSql('select count(*) from tbl where 重量 >= 50')
    dictSql['納期決定済み'] = executeSql('select count(*) from tbl where 納期 != ""')
    pprint(dictSql)


    with open('template.html', 'r', encoding='utf-8') as file:
        html = file.read()

    template = Template(html)
    htmlReplaced = template.render(dictSql)
    htmlTable = extract_div_content(htmlReplaced)

    dictView = getViews()
    views = extractHtml(dictView,  htmlTable)
    updateView(views)
    app_deploy()

    print('---------完了---------')
